@@ -66,13 +66,18 @@ class OpenEdxPublicSignal(Signal):
         """
         return cls._mapping[event_type]
 
-    def generate_signal_metadata(self):
+    def generate_signal_metadata(self, time=None):
         """
         Generate signal metadata when an event is sent.
 
         These fields are generated on the fly and are a subset of the Event
         Message defined in the OEP-41.
 
+        Arguments:
+            time (datetime): (Optional) Timestamp when the event was sent with
+                UTC timezone. Defaults to current time in UTC. See OEP-41 for
+                details.
+
         Example usage:
             >>> metadata = \
                 STUDENT_REGISTRATION_COMPLETED.generate_signal_metadata()
@@ -90,38 +95,20 @@ class OpenEdxPublicSignal(Signal):
         return EventsMetadata(
             event_type=self.event_type,
             minorversion=self.minor_version,
+            time=time,
         )
 
-    def send_event(self, send_robust=True, **kwargs):
+    def _send_event_with_metadata(self, metadata, send_robust=True, **kwargs):
         """
-        Send events to all connected receivers.
+        Send events to all connected receivers with the provided metadata.
 
-        Used to send events just like Django signals are sent. In addition,
-        some validations are executed on the arguments, and then generates relevant
-        metadata that can be used for logging or debugging purposes. Besides this behavior,
-        send_event behaves just like the send method.
+        This method is for internal use only.
 
-        If the event is disabled (i.e _allow_events is False), then this method
-        won't have any effect. Meaning, the Django Signal won't be sent.
+        Arguments:
+            metadata (EventsMetadata): The metadata to be sent with the signal.
+            send_robust (bool): Defaults to True. See Django signal docs.
 
-        Example usage:
-            >>> STUDENT_REGISTRATION_COMPLETED.send_event(
-                user=user_data, registration=registration_data,
-            )
-            [(<function callback at 0x7f2ce638ef70>, 'callback response')]
-
-        Keyword arguments:
-            send_robust (bool): determines whether the Django signal will be
-            sent using the method `send` or `send_robust`.
-
-        Returns:
-            list: response of each receiver following the format
-            [(receiver, response), ... ]. Empty list if the event is disabled.
-
-        Exceptions raised:
-            SenderValidationError: raised when there's a mismatch between
-            arguments passed to this method and arguments used to initialize
-            the event.
+        See ``send_event`` docstring for more details on its usage and behavior.
         """
 
         def validate_sender():
@@ -158,7 +145,7 @@ class OpenEdxPublicSignal(Signal):
 
         validate_sender()
 
-        kwargs["metadata"] = self.generate_signal_metadata()
+        kwargs["metadata"] = metadata
 
         if self._allow_send_event_failure or settings.DEBUG or not send_robust:
             return super().send(sender=None, **kwargs)
@@ -170,6 +157,86 @@ class OpenEdxPublicSignal(Signal):
 
         return responses
 
+    def send_event(self, send_robust=True, time=None, **kwargs):
+        """
+        Send events to all connected receivers.
+
+        Arguments:
+            send_robust (bool): Defaults to True. See Django signal docs.
+            time (datetime): (Optional - see note) Timestamp when the event was
+                sent with UTC timezone. For events requiring a DB create or
+                update, use the timestamp from the DB record. Defaults to
+                current time in UTC. This argument is optional for backward
+                compatability, but ideally would be explicitly set. See OEP-41
+                for details.
+
+        Used to send events just like Django signals are sent. In addition,
+        some validations are executed on the arguments, and then generates relevant
+        metadata that can be used for logging or debugging purposes. Besides this behavior,
+        send_event behaves just like the send method.
+
+        If the event is disabled (i.e _allow_events is False), then this method
+        won't have any effect. Meaning, the Django Signal won't be sent.
+
+        Example usage:
+            >>> STUDENT_REGISTRATION_COMPLETED.send_event(
+                user=user_data, registration=registration_data,
+            )
+            [(<function callback at 0x7f2ce638ef70>, 'callback response')]
+
+        Keyword arguments:
+            send_robust (bool): determines whether the Django signal will be
+            sent using the method `send` or `send_robust`.
+
+        Returns:
+            list: response of each receiver following the format
+            [(receiver, response), ... ]. Empty list if the event is disabled.
+
+        Exceptions raised:
+            SenderValidationError: raised when there's a mismatch between
+            arguments passed to this method and arguments used to initialize
+            the event.
+        """
+        metadata = self.generate_signal_metadata(time=time)
+        return self._send_event_with_metadata(metadata=metadata, send_robust=send_robust, **kwargs)
+
+    def send_event_with_custom_metadata(
+        self, *, id, minorversion, source, sourcehost, time, sourcelib,  # pylint: disable=redefined-builtin
+        send_robust=True, **kwargs
+    ):
+        """
+        Send events to all connected receivers using the provided metadata.
+
+        This method works exactly like ``send_event``, except it takes most of
+            the event metadata fields as arguments. This could be used for an
+            event bus consumer, where we want to recreate the metadata used
+            in the producer when resending the same signal on the consuming
+            side.
+
+        Arguments:
+            id (UUID): from event production metadata.
+            event_type (str): from event production metadata.
+            minorversion (int): from event production metadata.
+            source (str): from event production metadata.
+            sourcehost (str): from event production metadata.
+            time (datetime): from event production metadata.
+            sourcelib (tuple of ints): from event production metadata.
+            send_robust (bool): Defaults to True. See Django signal docs.
+
+        See ``send_event`` docstring for more details.
+
+        """
+        metadata = EventsMetadata(
+            id=id,
+            event_type=self.event_type,
+            minorversion=minorversion,
+            source=source,
+            sourcehost=sourcehost,
+            time=time,
+            sourcelib=sourcelib,
+        )
+        return self._send_event_with_metadata(metadata=metadata, send_robust=send_robust, **kwargs)
+
     def send(self, sender, **kwargs):  # pylint: disable=unused-argument
         """
         Override method used to recommend the sender to adopt our custom send.
